COMMENT

NMDAghk is basically a modification of Exp2Syn (see nrn/src/nrnoc/exp2syn.mod) with GHK IV relationship from nrn/share/examples/nrniv/nmodl/cachan.mod

According to Pickard 1985, single channel permeability is in m^3 / s (MKS units).
This is way too big for a single channel, since a single activation of an NMDAergic synapse that drives permeability to a maximum of 0.5e-19 m3/s is enough
to produce a 0.6 mV depolarization of a single compartment model with surface area 100 um2, g_pas 1e-4 S/cm2 (membrane time constant 10 ms), and resting potential -70 mV.
um3/s or um3/ms is more practical.

Ionic concentrations are from ghknmda.mod at modeldb.science/244848
Basak R, Narayanan R. Active dendrites regulate the spatiotemporal spread of signaling microdomains. PLoS Comput Biol. 14:e1006485, 2018. doi: 10.1371/journal.pcbi.1006485. PMID: 30383745.

PNMDA/PAMPA ratio (NAR, 1.5) and dynamics of permeability transient are from 
Narayanan R, Johnston D. The h current is a candidate mechanism for regulating the sliding modification threshold in a BCM-like synaptic learning rule. Journal of Neurophysiology 104: 1020–1033, 2010. PMID: 20554832.

PCa/PNa ratio (PCaPNaratio, 10.6) for NMDA-R gated channels is from
Mayer ML, Westbrook GL. Permeation and block of N-methyl-D-aspartic acid receptor channels by divalent cations in mouse cultured central neurones. Journal of Physiology 394: 501–527, 1987. PMID: 2451020.

mg block is from
Jahr CE, Stevens CF. Voltage dependence of NMDA-activated macroscopic conductances predicted by single-channel kinetics. J Neurosci. 10:3178-82, 1990. doi: 10.1523/JNEUROSCI.10-09-03178.1990. PMID: 1697902.

Note that this mechanism's GHK formulas use celsius, 
and that NEURON's default value for celsius is 6.3 degC.
Use a hoc or Python statement (celsius = foo or h.celsius = foo)
to specify the correct operating temperature for your model.

-------

From exp2syn.mod:
Two state kinetic scheme synapse described by rise time tau1,
and decay time constant tau2. The normalized peak conductance is 1.
Decay time MUST be greater than rise time.

The solution of A->G->bath with rate constants 1/tau1 and 1/tau2 is
 A = a*exp(-t/tau1) and
 G = a*tau2/(tau2-tau1)*(-exp(-t/tau1) + exp(-t/tau2))
        where tau1 < tau2

If tau2-tau1 is very small compared to tau1, this is an alphasynapse with time constant tau2.
If tau1/tau2 is very small, this is single exponential decay with time constant tau2.

The factor is evaluated in the initial block 
such that an event of weight 1 generates a
peak conductance of 1.

Because the solution is a sum of exponentials, the
coupled equations can be solved as a pair of independent equations
by the more efficient cnexp method.
ENDCOMMENT

NEURON {
  : Use a unique name to distinguish it from previous implementations
  : which (correctly) addressed bath application of agonist
  : (activating many receptors over the surface of a dendrite),
  : or attempted representation of transmission at individual synapses
  : without dealing properly with permeability.
  : The name should start with the name of the transmitter/receptor,
  : which is the salient biological fact.
  : Its instantaneous voltage dependence is secondary.
  : Use customary case rule for class names.
  POINT_PROCESS NMDAghk
  USEION na WRITE ina
  USEION k WRITE ik
  USEION ca READ cai, cao WRITE ica

  RANGE ina, ik, ica : make ionic current components visible to hoc or Python
  RANGE i : make it easy to get net current generated by an individual synapse
  RANGE tau1, tau2

  RANGE P, mg
  GLOBAL mgb
}

UNITS {
  (nA) = (nanoamp)
  (mV) = (millivolt)
  (uS) = (microsiemens)
  (molar) = (1/liter)
  (mM) = (millimolar)
  FARADAY = (faraday) (coulomb)
  R = (k-mole) (joule/degC)
  (um) = (micron)
}

PARAMETER {
  tau1 = 5  (ms) <1e-9,1e9> : Narayanan & Johnston 2010
  tau2 = 50 (ms) <1e-9,1e9> : Narayanan & Johnston 2010
  cai = 50e-6 (mM) : 50 nM
  cao = 2   (mM)
  nai = 18  (mM) : so ena is +55mV
  nao = 140 (mM)
  ki = 140  (mM)  : so ek is -90mV
  ko = 5    (mM)
  celsius   (degC)
  mg = 2    (mM)
  PCaPNaratio = 10.6 : Mayer and Westbrook 1987
}

ASSIGNED {
  ina (nA)
  ik  (nA)
  ica (nA)
  i (nA)
  v (mV)
  P (um3/s)

  factor (1)
  mgb (1)
}

STATE {
  A (um3/s)
  B (um3/s)
}

INITIAL {
  LOCAL tp
  if (tau1/tau2 > 0.9999) {
    tau1 = 0.9999*tau2
  }
  A = 0
  B = 0
  tp = (tau1*tau2)/(tau2 - tau1) * log(tau2/tau1)
  factor = -exp(-tp/tau1) + exp(-tp/tau2)
  factor = 1/factor
}

BREAKPOINT {
  SOLVE state METHOD cnexp
  P = B - A
  mgb = mgblock(v)

  ina = (1e-6)*P*mgb*ghk(v, nai, nao,1)  
  ica = (1e-6)*P*PCaPNaratio*mgb*ghk(v, cai, cao,2)
  ik = (1e-6)*P*mgb*ghk(v, ki, ko,1)
  i = ica + ik + ina
}

DERIVATIVE state {
  A' = -A/tau1
  B' = -B/tau2
}

FUNCTION ghk(v(mV), ci(mM), co(mM),z) (0.001 coul/cm3) {
  LOCAL arg, eci, eco
  arg = (0.001)*z*FARADAY*v/(R*(celsius+273.15))
  eco = co*efun(arg)
  eci = ci*efun(-arg)
  ghk = (0.001)*z*FARADAY*(eci - eco)
}

FUNCTION efun(z) {
  if (fabs(z) < 1e-4) {
    efun = 1 - z/2
  }else{
    efun = z/(exp(z) - 1)
  }
}

FUNCTION mgblock(v(mV)) (1){
  TABLE 
  DEPEND mg
  FROM -140 TO 80 WITH 1000 
  : from Jahr & Stevens 1990
  mgblock = 1 / (1 + exp(0.062 (/mV) * -v) * (mg / 3.57 (mM)))
}

NET_RECEIVE(weight (um3/s)) {
  A = A + weight*factor
  B = B + weight*factor
}
